@@ -80,6 +80,7 @@
  *         Graphics pixel setting flags
  *         Size and location filter flags
  *         Color component selection flags
+ *         Color content flags
  *         16-bit conversion flags
  *         Rotation and shear flags
  *         Affine transform order flags
@@ -100,8 +101,9 @@
  *         Box size adjustment flags
  *         Flags for modifying box boundaries using a second box
  *         Handling overlapping bounding boxes in boxa
+ *         Selecting or making a box from two (intersecting) boxes
  *         Flags for replacing invalid boxes
- *         Flags for box corners
+ *         Flags for box corners and center
  *         Horizontal warp
  *         Pixel selection for resampling
  *         Thinning flags
@@ -421,8 +423,7 @@ enum {
  *
  *       (6) The version numbers (below) are used in the serialization
  *           of these data structures.  They are placed in the files,
- *           and rarely (if ever) change.  Provision is currently made for
- *           backward compatibility in reading from boxaa version 2.
+ *           and rarely (if ever) change.
  *
  *       (7) The serialization dependencies are as follows:
  *               pixaa  :  pixa  :  boxa
@@ -444,7 +445,6 @@ enum {
 /*-------------------------------------------------------------------------*
  *                              Array of pix                               *
  *-------------------------------------------------------------------------*/
-
     /*  Serialization for primary data structures */
 #define  PIXAA_VERSION_NUMBER      2  /*!< Version for Pixaa serialization */
 #define  PIXA_VERSION_NUMBER       2  /*!< Version for Pixa serialization  */
@@ -476,7 +476,6 @@ typedef struct Pixaa PIXAA;
 /*-------------------------------------------------------------------------*
  *                    Basic rectangle and rectangle arrays                 *
  *-------------------------------------------------------------------------*/
-
 /*! Basic rectangle */
 struct Box
 {
@@ -527,7 +526,6 @@ typedef struct Pta PTA;
 /*-------------------------------------------------------------------------*
  *                              Array of Pta                               *
  *-------------------------------------------------------------------------*/
-
 /*! Array of Pta */
 struct Ptaa
 {
@@ -541,7 +539,6 @@ typedef struct Ptaa PTAA;
 /*-------------------------------------------------------------------------*
  *                       Pix accumulator container                         *
  *-------------------------------------------------------------------------*/
-
 /*! Pix accumulator container */
 struct Pixacc
 {
@@ -557,7 +554,6 @@ typedef struct Pixacc PIXACC;
 /*-------------------------------------------------------------------------*
  *                              Pix tiling                                 *
  *-------------------------------------------------------------------------*/
-
 /*! Pix tiling */
 struct PixTiling
 {
@@ -628,7 +624,6 @@ typedef struct DPix DPIX;
 /*-------------------------------------------------------------------------*
  *                        PixComp: compressed pix                          *
  *-------------------------------------------------------------------------*/
-
 /*! Compressed Pix */
 struct PixComp
 {
@@ -710,7 +705,6 @@ typedef struct PixaComp PIXAC;
  *  of that string.
  * </pre>
  */
-
 /*! Object Access */
 enum {
     L_NOCOPY = 0,     /*!< do not copy the object; do not delete the ptr  */
@@ -721,7 +715,6 @@ enum {
                       /*!< the array with clones (e.g., pix)              */
 };
 
-
 /*----------------------------------------------------------------------------*
  *                              Sort flags                                    *
  *----------------------------------------------------------------------------*/
@@ -752,7 +745,6 @@ enum {
     L_SORT_BY_ASPECT_RATIO = 11  /*!< sort box or c.c. by width/height ratio  */
 };
 
-
 /*---------------------------------------------------------------------------*
  *                             Blend flags                                   *
  *---------------------------------------------------------------------------*/
@@ -772,7 +764,6 @@ enum {
     L_PAINT_DARK = 2              /*!< colorize non-white pixels             */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                        Graphics pixel setting                           *
  *-------------------------------------------------------------------------*/
@@ -783,7 +774,6 @@ enum {
     L_FLIP_PIXELS = 3           /*!< flip all bits in each pixel           */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                     Size and location filter flags                      *
  *-------------------------------------------------------------------------*/
@@ -824,7 +814,6 @@ enum {
     L_CHECK_BOTH = 3            /*!< check and possibly modify both        */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                    Color component selection flags                      *
  *-------------------------------------------------------------------------*/
@@ -837,9 +826,19 @@ enum {
     L_SELECT_MAX = 5,           /*!< use max color component               */
     L_SELECT_AVERAGE = 6,       /*!< use average of color components       */
     L_SELECT_HUE = 7,           /*!< use hue value (in HSV color space)    */
-    L_SELECT_SATURATION = 8     /*!< use saturation value (in HSV space)   */
+    L_SELECT_SATURATION = 8,    /*!< use saturation value (in HSV space)   */
+    L_SELECT_WEIGHTED = 9       /*!< use weighted average of color comps   */
 };
 
+/*-------------------------------------------------------------------------*
+ *                          Color content flags                            *
+ *-------------------------------------------------------------------------*/
+/*! Color Content */
+enum {
+    L_INTERMED_DIFF = 1,        /*!< intermediate of diff component values */
+    L_AVE_MAX_DIFF_2 = 2,       /*!< diff average closest comps to third   */
+    L_MAX_DIFF = 3              /*!< maximum diff of component values      */
+};
 
 /*-------------------------------------------------------------------------*
  *                         16-bit conversion flags                         *
@@ -855,7 +854,6 @@ enum {
     L_CLIP_TO_FFFF = 7          /*!< use max(val, 65535)                   */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                        Rotate and shear flags                           *
  *-------------------------------------------------------------------------*/
@@ -878,7 +876,6 @@ enum {
     L_SHEAR_ABOUT_CENTER = 2   /*!< shear image about center               */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                     Affine transform order flags                        *
  *-------------------------------------------------------------------------*/
@@ -892,7 +889,6 @@ enum {
     L_SC_TR_RO = 6             /*!< scale, translate, rotate               */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                       Grayscale filling flags                           *
  *-------------------------------------------------------------------------*/
@@ -902,7 +898,6 @@ enum {
     L_FILL_BLACK = 2          /*!< fill black pixels (e.g., in bg map)     */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                   Flags for setting to white or black                   *
  *-------------------------------------------------------------------------*/
@@ -912,7 +907,6 @@ enum {
     L_SET_BLACK = 2          /*!< set pixels to black                      */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                  Flags for getting white or black value                 *
  *-------------------------------------------------------------------------*/
@@ -922,7 +916,6 @@ enum {
     L_GET_BLACK_VAL = 2      /*!< get black pixel value                    */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                  Flags for 8 bit and 16 bit pixel sums                  *
  *-------------------------------------------------------------------------*/
@@ -932,7 +925,6 @@ enum {
     L_BLACK_IS_MAX = 2  /*!< black pixels are 0xff or 0xffff; white are 0  */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                           Dither parameters                             *
  *         If within this grayscale distance from black or white,          *
@@ -946,7 +938,6 @@ enum {
     DEFAULT_CLIP_UPPER_2 = 5   /*!< dist to black with no prop; 2 bpp      */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                          Distance type flags                            *
  *-------------------------------------------------------------------------*/
@@ -956,7 +947,6 @@ enum {
     L_EUCLIDEAN_DISTANCE = 2   /*!< L2 distance                            */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                         Distance Value flags                            *
  *-------------------------------------------------------------------------*/
@@ -970,7 +960,6 @@ enum {
     L_ALL = 6            /*!< all values                                   */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                         Statistical measures                            *
  *-------------------------------------------------------------------------*/
@@ -985,7 +974,6 @@ enum {
     L_VARIANCE = 7             /*!< variance of values                     */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                       Set index selection flags                         *
  *-------------------------------------------------------------------------*/
@@ -995,7 +983,6 @@ enum {
     L_CHOOSE_SKIP_BY = 2       /*!< select at intervals of 'n'             */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                         Text orientation flags                          *
  *-------------------------------------------------------------------------*/
@@ -1008,7 +995,6 @@ enum {
     L_TEXT_ORIENT_RIGHT = 4    /*!< landscape, text up to right            */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                         Edge orientation flags                          *
  *-------------------------------------------------------------------------*/
@@ -1019,7 +1005,6 @@ enum {
     L_ALL_EDGES = 2           /*!< filters for all edges                   */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                         Line orientation flags                          *
  *-------------------------------------------------------------------------*/
@@ -1032,7 +1017,6 @@ enum {
     L_OBLIQUE_LINE = 4       /*!< neither horizontal nor vertical */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                         Image orientation flags                         *
  *-------------------------------------------------------------------------*/
@@ -1042,7 +1026,6 @@ enum {
     L_LANDSCAPE_MODE = 1   /*!< page is viewed at 90 deg to portrait mode    */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                           Scan direction flags                          *
  *-------------------------------------------------------------------------*/
@@ -1059,7 +1042,6 @@ enum {
     L_SCAN_VERTICAL = 8      /*!< vertical scan (direction unimportant)    */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                Box size adjustment and location flags                   *
  *-------------------------------------------------------------------------*/
@@ -1084,7 +1066,6 @@ enum {
     L_GET_BOT = 16               /*!< get bottom side location             */
 };
 
-
 /*-------------------------------------------------------------------------*
  *          Flags for modifying box boundaries using a second box          *
  *-------------------------------------------------------------------------*/
@@ -1098,7 +1079,6 @@ enum {
     L_USE_CAPPED_MAX = 6         /*!< modify boundary with capped max      */
 };
 
-
 /*-------------------------------------------------------------------------*
  *              Handling overlapping bounding boxes in boxa                *
  *-------------------------------------------------------------------------*/
@@ -1108,6 +1088,16 @@ enum {
     L_REMOVE_SMALL = 2     /*!< only remove smaller                        */
 };
 
+/*-------------------------------------------------------------------------*
+ *        Selecting or making a box from two (intersecting) boxes          *
+ *-------------------------------------------------------------------------*/
+/*! Box Combine or Select */
+enum {
+    L_GEOMETRIC_UNION = 1,         /*!< use union of two boxes             */
+    L_GEOMETRIC_INTERSECTION = 2,  /*!< use intersection of two boxes      */
+    L_LARGEST_AREA = 3,            /*!< use box with largest area          */
+    L_SMALLEST_AREA = 4            /*!< use box with smallest area         */
+};
 
 /*-------------------------------------------------------------------------*
  *                    Flags for replacing invalid boxes                    *
@@ -1118,19 +1108,18 @@ enum {
     L_USE_SAME_PARITY_BOXES = 2  /*!< consider boxes with the same parity  */
 };
 
-
 /*-------------------------------------------------------------------------*
- *                          Flags for box corners                          *
+ *                    Flags for box corners and center                     *
  *-------------------------------------------------------------------------*/
-/*! Box Corners */
+/*! Box Corners and Center */
 enum {
     L_UPPER_LEFT = 1,         /*!< UL corner                               */
     L_UPPER_RIGHT = 2,        /*!< UR corner                               */
     L_LOWER_LEFT = 3,         /*!< LL corner                               */
-    L_LOWER_RIGHT = 4         /*!< LR corner                               */
+    L_LOWER_RIGHT = 4,        /*!< LR corner                               */
+    L_BOX_CENTER = 5          /*!< center                                  */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                            Horizontal warp                              *
  *-------------------------------------------------------------------------*/
@@ -1146,7 +1135,6 @@ enum {
     L_QUADRATIC_WARP = 2   /*!< stretch or contraction grows quadratically */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                      Pixel selection for resampling                     *
  *-------------------------------------------------------------------------*/
@@ -1156,7 +1144,6 @@ enum {
     L_SAMPLED = 2          /*!< nearest src pixel sampling only            */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                             Thinning flags                              *
  *-------------------------------------------------------------------------*/
@@ -1166,7 +1153,6 @@ enum {
     L_THIN_BG = 2              /*!< thin background of 1 bpp image         */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                            Runlength flags                              *
  *-------------------------------------------------------------------------*/
@@ -1176,7 +1162,6 @@ enum {
     L_VERTICAL_RUNS = 1      /*!< determine runlengths of vertical runs    */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                          Edge filter flags                              *
  *-------------------------------------------------------------------------*/
@@ -1186,7 +1171,6 @@ enum {
     L_TWO_SIDED_EDGE = 2     /*!< Two-sided edge filter                    */
 };
 
-
 /*-------------------------------------------------------------------------*
  *             Subpixel color component ordering in LCD display            *
  *-------------------------------------------------------------------------*/
@@ -1198,7 +1182,6 @@ enum {
     L_SUBPIXEL_ORDER_VBGR = 4   /*!< sensor order top-to-bottom BGR        */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                          HSV histogram flags                            *
  *-------------------------------------------------------------------------*/
@@ -1209,7 +1192,6 @@ enum {
     L_SV_HISTO = 3             /*!< Use saturation-value histogram         */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                HSV Region flags (inclusion, exclusion)                  *
  *-------------------------------------------------------------------------*/
@@ -1219,7 +1201,6 @@ enum {
     L_EXCLUDE_REGION = 2       /*!< Use pixels outside HSV region          */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                Location flags for adding text to a pix                  *
  *-------------------------------------------------------------------------*/
@@ -1235,7 +1216,6 @@ enum {
     L_ADD_AT_RIGHT = 8         /*!< Add text over right side of the image  */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                       Flags for plotting on a pix                       *
  *-------------------------------------------------------------------------*/
@@ -1249,7 +1229,6 @@ enum {
     L_PLOT_AT_RIGHT = 6        /*!< Plot vertically at right               */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                    Flags for making simple masks                        *
  *-------------------------------------------------------------------------*/
@@ -1259,7 +1238,6 @@ enum {
     L_USE_OUTER = 2            /*!< Select the outer part (e.g., a frame)  */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                   Flags for selecting display program                   *
  *-------------------------------------------------------------------------*/
@@ -1283,7 +1261,6 @@ enum {
     L_NO_CHROMA_SAMPLING_JPEG = 1   /*!< Write full resolution chroma      */
 };
 
-
 /*-------------------------------------------------------------------------*
  *          Handling negative values in conversion to unsigned int         *
  *-------------------------------------------------------------------------*/
@@ -1293,7 +1270,6 @@ enum {
     L_TAKE_ABSVAL = 2        /*!< Convert to positive using L_ABS()        */
 };
 
-
 /*-------------------------------------------------------------------------*
  *                        Relative to zero flags                           *
  *-------------------------------------------------------------------------*/
@@ -1304,7 +1280,6 @@ enum {
     L_GREATER_THAN_ZERO = 3  /*!< Choose values greater than zero          */
 };
 
-
 /*-------------------------------------------------------------------------*
  *         Flags for adding or removing trailing slash from string         *
  *-------------------------------------------------------------------------*/
@@ -1314,7 +1289,6 @@ enum {
     L_REMOVE_TRAIL_SLASH = 2   /*!< Remove trailing slash from string      */
 };
 
-
 /*-------------------------------------------------------------------------*
  *               Pix allocator and deallocator function types              *
  *-------------------------------------------------------------------------*/
@@ -1324,5 +1298,4 @@ typedef void *(*alloc_fn)(size_t);
 /*! Deallocator function type */
 typedef void (*dealloc_fn)(void *);
 
-
 #endif  /* LEPTONICA_PIX_H */
